/*-----------------------------------------------------------------------
censoring_simulation.do

Stata do file used to recreate the panel for
'Selection in initial and return migration',
by Jorge De la Roca,
Journal of Urban Economics

Simulates MCVL earnings following Card, Heining and Kline, QJE 2013
-----------------------------------------------------------------------*/

** Prepare MCVL 2013 cohorts for earnings simulation
quietly {
use "../../tmp/cohort_2013", clear

* Foreign born
keep if birth_country == "N00" & birth_prov >= 1 & birth_prov <= 52
* Foreign nationals
drop if nationality != "N00" & nationality != ""
drop nationality birth_country
* Missing social security registration province
drop if ss_reg_prov == . | ss_reg_prov == 0
* Missing level of education
replace	edu_code = "100" if edu_code == "C0"
replace	edu_code = "100" if edu_code == "[B"
destring edu_code, replace
* set coding errors to missing
replace edu_code = . if (edu_code >  0 & edu_code < 10) | (edu_code > 11 & edu_code < 20)
replace edu_code = . if (edu_code > 22 & edu_code < 30) | (edu_code > 32 & edu_code < 40) | edu_code > 48
drop if edu_code == .
* Ages below 18 or above 65
gen age = (year + month/12) - (int(birth_date/100) + (birth_date - 100*int(birth_date/100))/12)
drop if age < 18 | age > 65
* Years before 1980
drop if year < 1980

* Keep general SS regime and self-employed
keep if regime == 111 | regime == 521
* Drop non-contributive occupations
drop if occupation == 13 | occupation == 19
drop if occupation == 0 & regime == 111
* Recode unusual occupations
recode occupation 12=11
replace occupation = 0 if regime == 521 & occupation != 0
* Peculiar job_relationships 
drop if job_relationship == 400
* Missing workplace province
gen prov_id = int(muni_id / 1000)
replace prov_id = . if prov_id == 0
drop if prov_id == .
* Missing number of work days
drop if days_worked_month == .
* Negative earnings and outliers
drop if earnings_capped_ss == 9999999

* Drop individuals with low labour force attachment (worked less than six months in any year)
egen days_year = sum(days_worked_month), by (person_id year)
egen max_days = max(days_year), by (person_id)
drop if max_days <= 183
compress
drop mcvl_wave birth_prov ss_reg_prov experience tenure uexp_1_2 - uexp_smaller regime ptcoef - sector firm_id_sec sector_cnae93 prov_id - max_days
save "../../tmp/temp_cohort_2013", replace


** Add MCVL additional waves (2005 - 2012)
forval x = 2005(1)2012 {
use "../../tmp/additional_`x'", clear

* Keep individuals who were born after the earliest cohort of analysis (see above)
drop if birth_date < 191600
* Keep individuals who were born before the latest cohort of analysis (see above)
drop if birth_date > 199500

* Foreign born
keep if birth_country == "N00" & birth_prov >= 1 & birth_prov <= 52
* Foreign nationals
drop if nationality != "N00" & nationality != ""
drop nationality birth_country
* Missing social security registration province
drop if ss_reg_prov == . | ss_reg_prov == 0
* Missing level of education
replace	edu_code = "100" if edu_code == "C0"
replace	edu_code = "100" if edu_code == "[B"
destring edu_code, replace
* set coding errors to missing
replace edu_code = . if (edu_code >  0 & edu_code < 10) | (edu_code > 11 & edu_code < 20)
replace edu_code = . if (edu_code > 22 & edu_code < 30) | (edu_code > 32 & edu_code < 40) | edu_code > 48
drop if edu_code == .
* Ages below 18 or above 65
gen age = (year + month/12) - (int(birth_date/100) + (birth_date - 100*int(birth_date/100))/12)
drop if age < 18 | age > 65
* Years before 1980
drop if year < 1980

* Keep general SS regime and self-employed
keep if regime == 111 | regime == 521
* Drop non-contributive occupations
drop if occupation == 13 | occupation == 19
drop if occupation == 0 & regime == 111
* Recode unusual occupations
recode occupation 12=11
replace occupation = 0 if regime == 521 & occupation != 0
* Peculiar job_relationships 
drop if job_relationship == 400
* Missing workplace province
gen prov_id = int(muni_id / 1000)
replace prov_id = . if prov_id == 0
drop if prov_id == .
* Missing number of work days
drop if days_worked_month == .
* Negative earnings and outliers
drop if earnings_capped_ss == 9999999

* Drop individuals with low labour force attachment (worked less than six months in any year)
egen days_year = sum(days_worked_month), by (person_id year)
egen max_days = max(days_year), by (person_id)
drop if max_days <= 183
compress
drop mcvl_wave birth_prov ss_reg_prov experience tenure uexp_1_2 - uexp_smaller regime ptcoef muni_id sector* firm_id_sec prov_id - max_days
save "../../tmp/temp_add_`x'", replace
}
}

** Combine all MCVL 2013 cohort databases and MCVL additional waves
use "../../tmp/temp_cohort_2013", clear
append using "../../tmp/temp_add_2012"
append using "../../tmp/temp_add_2011"
append using "../../tmp/temp_add_2010"
append using "../../tmp/temp_add_2009"
append using "../../tmp/temp_add_2008"
append using "../../tmp/temp_add_2007"
append using "../../tmp/temp_add_2006"
append using "../../tmp/temp_add_2005"

** Identify lower and upper caps 
merge m:1 year occupation using "../../otherdata/ss_bounds_1980_2013", keepusing(lower_cap1 upper_cap1)
drop if _merge == 2
drop _merge
merge m:1 year month using "../../otherdata/cpi_monthly", keepusing(cpi_index2013)
drop if _merge == 2
drop _merge
gen daysmonth = 30 if month == 4 | month == 6 | month == 9 | month == 11
replace daysmonth = 31 if month == 1 | month == 3 | month == 5 | month == 7 | month == 8 | month == 10 | month == 12
replace daysmonth = 28 if month == 2
replace daysmonth = 29 if month == 2 & (year == 1980 | year == 1984 | year == 1988 | year == 1992 | year == 1996 | year == 2000 | year == 2004 | year == 2008 | year == 2012)
* Identify upper censored incomes by year, month and occupation
bys year month occupation: egen p50 = pctile(earnings_capped_ss) if capped_top == 1
bys year month occupation: egen earnings_cap_top = min(p50)
replace earnings_cap_top = upper_cap1 if earnings_cap_top == .
drop p50
* Identify lower censored incomes by year, month and occupation
bys year month occupation: egen p50 = pctile(earnings_capped_ss) if capped_bottom == 1
bys year month occupation: egen earnings_cap_bot = min(p50)
replace earnings_cap_bot = lower_cap1 if earnings_cap_bot == .
drop p50

** Construct daily earnings variable with capped observations
* Daily earnings
gen earnings_capped_day = earnings_capped_ss / days_worked_month_ss
gen earnings_top_day = earnings_cap_top / daysmonth
gen earnings_bot_day = earnings_cap_bot / daysmonth
gen earnings_day = earnings_capped_day
replace earnings_day = earnings_top_day if earnings_capped_day >= earnings_top_day & earnings_top_day != .
replace earnings_day = earnings_top_day if capped_top == 1 & earnings_capped_day < earnings_top_day & earnings_top_day != .
replace earnings_day = earnings_bot_day if earnings_capped_day <= earnings_bot_day & earnings_bot_day != .
replace earnings_day = earnings_bot_day if capped_bottom == 1 & earnings_capped_day > earnings_bot_day & earnings_bot_day != .
* Construct censoring indicators
gen cap_top_day = 1 if earnings_day == earnings_top_day
replace cap_top_day = 0 if cap_top_day == .
gen cap_bot_day = 1 if earnings_day == earnings_bot_day
replace cap_bot_day = 0 if cap_bot_day == .
* Deflate daily earnings using monthly CPI index 
replace earnings_top_day = earnings_top_day / cpi_index2013
replace earnings_bot_day = earnings_bot_day / cpi_index2013
replace earnings_day = earnings_day / cpi_index2013
drop lower_cap1 upper_cap1 daysmonth - earnings_capped_day

** Merge with earnings from tax data
merge m:1 person_id year firm_id_tax using "../../tmp/tax_all", keepusing(earnings)
drop if _merge == 2
rename _merge merge_mcvltax
rename earnings earnings_year_firm_tax
replace earnings_capped_ss = earnings_capped_ss / cpi_index2013
* Compute ratio of tax to social security earnings in the year 
sort person_id year month 
egen days_year_firm = sum(days_worked_month_ss) if merge_mcvltax == 3, by (person_id year firm_id_tax)
egen earnings_year_firm_ss = sum(earnings_capped_ss) if merge_mcvltax == 3, by (person_id year firm_id_tax)
egen min_earnings_firm = min(earnings_capped_ss) if merge_mcvltax == 3, by (person_id year firm_id_tax)
gen ratio = earnings_year_firm_tax / earnings_year_firm_ss
* Daily earnings (annual tax earnings in the firm allocated across months in the firm in proportion to social security earnings)
gen earnings_day_tax = earnings_capped_ss * ratio / days_worked_month_ss if min_earnings_firm >= 0
replace earnings_day_tax = earnings_year_firm_tax / days_year_firm if min_earnings_firm < 0
drop cpi_index2013 earnings_year_firm_tax - ratio

** Create mean earnings and incidence of censoring for each individual over his lifetime
* Two specifications: with and without tax data 
sort person_id year month
gen uno = 1 
gen top = cap_top_day
gen bot = cap_bot_day
egen obs_num = sum(uno), by (person_id)
egen obs_top = sum(top), by (person_id)
egen obs_bot = sum(bot), by (person_id)
* Construct worker's mean of log earnings over lifetime (except for current period)
gen ln_earnings = ln(earnings_day)
egen ln_sum = sum(ln_earnings), by (person_id)
gen ln_mean = (ln_sum - ln_earnings) / (obs_num - 1)
* Construct worker's share of censored observations over lifetime (except for current period)
gen share_top = (obs_top - top) / (obs_num - 1)
gen share_bot = (obs_bot - bot) / (obs_num - 1)
* Construct worker's mean of log earnings from tax data (except for current period)
gen ln_tax = ln(earnings_day_tax)
egen ln_mean_tax = mean(ln_tax), by (person_id)
replace ln_mean_tax = 0 if ln_mean_tax == .
gen non_tax = 1 if ln_mean_tax == 0
replace non_tax = 0 if non_tax == .
drop earnings_capped_ss - capped_top uno - ln_sum ln_tax

** Create contract_type dummies
gen contract_pt = 1
replace contract_pt = 0 if (contract_type >=  0  & contract_type <=   2) | (contract_type >=   7 & contract_type <=  17) | (contract_type >= 19 & contract_type <= 22)
replace contract_pt = 0 if (contract_type >= 28  & contract_type <=  33) | (contract_type >=  35 & contract_type <=  37) | (contract_type >= 39 & contract_type <= 62)
replace contract_pt = 0 if (contract_type >= 66  & contract_type <=  72) | (contract_type >=  77 & contract_type <=  80) | contract_type == 82
replace contract_pt = 0 if (contract_type >= 85  & contract_type <=  88) | (contract_type >=  90 & contract_type <=  92) | (contract_type >= 96 & contract_type <= 97)
replace contract_pt = 0 if (contract_type >= 100 & contract_type <= 101) | (contract_type >= 103 & contract_type <= 180) | contract_type == 189
replace contract_pt = 0 if (contract_type >= 401 & contract_type <= 457)
gen contract_temp = 1
replace contract_temp = 0 if (contract_type >=   0  & contract_type <=   3) | (contract_type >=   8 & contract_type <=   9) | (contract_type >=  11 & contract_type <=  13)
replace contract_temp = 0 if (contract_type >=  18  & contract_type <=  21) | (contract_type >=  23 & contract_type <=  23) | (contract_type >=  28 & contract_type <=  29)
replace contract_temp = 0 if (contract_type >=  33  & contract_type <=  33) | (contract_type >=  35 & contract_type <=  35) | (contract_type >=  38 & contract_type <=  51)
replace contract_temp = 0 if (contract_type >=  59  & contract_type <=  63) | (contract_type >=  65 & contract_type <=  65) | (contract_type >=  69 & contract_type <=  71)
replace contract_temp = 0 if (contract_type >=  80  & contract_type <=  81) | (contract_type >=  86 & contract_type <=  86) | (contract_type >=  88 & contract_type <=  91)
replace contract_temp = 0 if (contract_type >=  98  & contract_type <=  98) | (contract_type >= 100 & contract_type <= 189) | (contract_type >= 200 & contract_type <= 289)
replace contract_temp = 0 if (contract_type >= 300  & contract_type <= 389) 

** Bring measure of city size and calculate mean earnings in city by year
merge m:1 ua_id using "../../otherdata/ua_size", keepusing(size_pop10k)
drop if _merge == 2
gen ln_size = ln(size_pop10k)
* Construct mean earnings in city by year
gen uno = 1 if ua_id != .
egen obs_num = sum(uno) if uno == 1, by (ua_id year)
gen ln_earnings = ln(earnings_day)
egen ln_sum = sum(ln_earnings) if uno == 1, by (ua_id year)
gen ln_city_mean = (ln_sum - ln_earnings) / (obs_num - 1)

** Ordering and labeling
drop days_worked_month - ua_id contract_type firm_id_tax size_pop10k _merge uno - ln_sum
compress
order person_id - month sex birth_date age edu_code 
sort person_id year month
label var age "Age"
label var earnings_top_day "Upper bound of daily real earnings for SS contributions in cents"
label var earnings_bot_day "Lower bound of daily real earnings for SS contributions in cents"
label var earnings_day "Daily real earnings recorded for SS contributions (possibly capped) in cents"
label var cap_top_day "Daily real earnings for SS contribution is top-capped"
label var cap_bot_day "Daily real earnings for SS contribution is bottom-capped"
label var earnings_day_tax "Daily real earnings from tax data (uncensored)"
label var ln_mean "Mean of worker's lifetime log earnings"
label var share_top "Share of worker's lifetime top-capped earnings observations"
label var share_bot "Share of worker's lifetime bottom-capped earnings observations"
label var ln_mean_tax "Mean of worker's lifetime tax earnings (2004-2013)"
label var non_tax "No information on tax earnings = 1"
label var contract_temp "Fixed-term contract"
label var contract_pt "Part-time contract"
label var ln_size "Log city size"
label var ln_city_mean "Mean of log earnings in city by year"
save "../../tmp/censoring_cohorts", replace

** Estimate Tobits to obtain means-variances for every occupation-year-age group combination
* Sex
replace sex = 0 if sex == 2
* Age groups 
gen age_group = 1 if age >= 18 & age < 25
replace age_group = 2 if age >= 25 & age < 30
replace age_group = 3 if age >= 30 & age < 35
replace age_group = 4 if age >= 35 & age < 40
replace age_group = 5 if age >= 40 & age < 45
replace age_group = 6 if age >= 45 & age < 50
replace age_group = 7 if age >= 50 & age < 55
replace age_group = 8 if age >= 55 & age != .
* Education categories
gen edu_belowsecondary = .
replace edu_belowsecondary = 1 if edu_code <= 32
replace edu_belowsecondary = 0 if edu_code > 32 & !missing(edu_code)
gen edu_secondary = .
replace edu_secondary = 1 if edu_code >= 40 & edu_code  <= 43
replace edu_secondary = 0 if (edu_code >= 44 & !missing(edu_code)) | edu_belowsecondary == 1
gen edu_tertiary = .
replace edu_tertiary = 1 if edu_code >= 44 & !missing(edu_code)
replace edu_tertiary = 0 if edu_belowsecondary == 1 | edu_secondary == 1
drop edu_code edu_belowsecondary
* Censoring indicator variable
gen capped = -1 if cap_bot_day == 1
replace capped = 1 if cap_top_day == 1
replace capped = 0 if capped == .
* Dependent variable
gen ln_earnings = ln(earnings_day)
* Drop unemployment spells
drop if job_relationship >= 751 & job_relationship <= 756
compress
tempfile tmp_init
save "`tmp_init'", replace

quietly {
local i = 1980
while `i' <= 2013 {
  use "`tmp_init'", clear
  keep if year == `i'

  * Month dummies
  xi i.month, noomit
  rename _I* *  

  local a = 1
  while `a' <= 8 {
    local o = 1
    while `o' <= 10 {

      * Mean and variance prediction	    
      local fullcontrols "ln_mean share_top share_bot sex age edu_secondary edu_tertiary contract_temp contract_pt ln_size ln_city_mean month_*"  
      cnreg ln_earnings `fullcontrols' if age_group == `a' & occupation == `o', censored(capped)
      predict xb_`a'_`o' if e(sample)
      matrix eb_`a'_`o' = e(b)
      gen col_`a'_`o' = colsof(eb_`a'_`o')
      gen sigma_`a'_`o' = eb_`a'_`o'[1,col_`a'_`o'] if e(sample)
      drop col_`a'_`o'
      matrix drop eb_`a'_`o'

      * Mean and variance prediction using also earnings from tax data
      local fullcontrols "ln_mean share_top share_bot ln_mean_tax non_tax sex age edu_secondary edu_tertiary contract_temp contract_pt ln_size ln_city_mean month_*"  
      cnreg ln_earnings `fullcontrols' if age_group == `a' & occupation == `o', censored(capped)
      predict xbfull_`a'_`o' if e(sample)
      matrix eb_`a'_`o' = e(b)
      gen col_`a'_`o' = colsof(eb_`a'_`o')
      gen sigmafull_`a'_`o' = eb_`a'_`o'[1,col_`a'_`o'] if e(sample)
      drop col_`a'_`o'
      matrix drop eb_`a'_`o'

      local o = `o' + 1
      }     
    local a = `a' + 1
    }

  * Combine all mean and variance predictions for all occupations in a calendar year
  egen xb = rmin(xb_*)
  egen xbfull = rmin(xbfull*)  
  egen sigma = rmin(sigma_*)
  egen sigmafull = rmin(sigmafull*)  
  drop month_* xb_* xbfull_* sigma_* sigmafull_*
  compress

  keep person_id year month earnings_top_day earnings_bot_day capped - sigmafull
  label var capped "Censored earnings observation (1=Top, 0= Uncensored, -1=Bottom)"
  label var ln_earnings "Log of real daily earnings (possibly capped)"
  label var xb "Log of mean earnings prediction by occup-year-age group"
  label var xbfull "Log of mean earnings prediction by occup-year-age group (including tax earnings)"  
  label var sigma "Variance of log earnings prediction by occup-year-age group"
  label var sigmafull "Variance of log earnings prediction by occup-year-age group (including tax earnings)"
  save "../../tmp/mean_var_`i'", replace

  local i = `i' + 1
}
} 

** Append all annual data sets with simulated earnings
use "../../tmp/mean_var_1980", clear
forvalues x = 1981(1)2013 {
 append using "../../tmp/mean_var_`x'"
}
* Restrict sample to available earnings
drop if xb == .
gen ln_unc = .
gen ln_uncfull = .
* Generate shocks using censoring bounds
set seed 738469
gen ln_top = ln(earnings_top_day)
gen ln_bot = ln(earnings_bot_day)
gen norm_top = normal((ln_top - xb) / sigma)
gen norm_bot = normal((ln_bot - xb) / sigma)
gen norm_topfull = normal((ln_top - xbfull) / sigmafull)
gen norm_botfull = normal((ln_bot - xbfull) / sigmafull)
gen shock1_top = invnormal(norm_top + uniform()*(1-norm_top))
replace shock1_top = 8.21 if shock1_top == .  
gen shock1_bot = invnormal(norm_bot * uniform())
replace shock1_bot = -8.21 if shock1_bot == .
gen shock2_top = invnormal(norm_topfull + uniform()*(1-norm_topfull))
replace shock2_top = 8.21 if shock2_top == .
gen shock2_bot = invnormal(norm_botfull * uniform())
replace shock2_bot = -8.21 if shock2_bot == .
* Simulate earnings specifications
replace ln_unc = xb + sigma * shock1_top if capped == 1
replace ln_unc = xb + sigma * shock1_bot if capped == -1
replace ln_uncfull = xbfull + sigmafull * shock2_top if capped == 1  
replace ln_uncfull = xbfull + sigmafull * shock2_bot if capped == -1
keep person_id year month capped* ln_earnings ln_unc ln_uncfull
* Label variables
label var capped "Censored earnings observation (1=Top, 0=Uncensored, -1=Bottom)"
label var ln_earnings "Log of real daily earnings (possibly capped)"
label var ln_unc "Log of simulated earnings"
label var ln_uncfull "Log of simulated earnings (+ tax earnings)"
save "../../tmp/censoring_simulation", replace

* Erase temporary data sets
erase "../../tmp/temp_cohort_2013.dta"
forvalues x = 2005(1)2012 {
erase "../../tmp/temp_add_`x'.dta"
}
forvalues x = 1980(1)2013 {
erase "../../tmp/mean_var_`x'.dta"
}
